@@ -7,6 +7,7 @@ pub struct JsonSolver {
     pub recursive: bool,
     pub json_line: bool,
     pub skip_empty: bool,
+    pub skip_keys: Vec<String>,
 }
 
 impl From<&clap::ArgMatches<'_>> for JsonSolver {
@@ -14,18 +15,25 @@ impl From<&clap::ArgMatches<'_>> for JsonSolver {
         let expression = input
             .value_of("expression")
             // TODO: I am pretty sure its perfectly legal to use "." as a value key in JSON?
-            .map(|s| s.split(".").map(String::from).collect::<_>())
+            .map(|s| s.split('.').map(String::from).collect::<_>())
             .unwrap_or_default();
         let pretty = input.is_present("pretty");
         let recursive = input.is_present("recursive");
         let json_line = input.is_present("json-lines");
         let skip_empty = input.is_present("skip-empty");
+        let skip_keys = input
+            .values_of("skip-key")
+            .unwrap_or_default()
+            .map(|s| s.to_string())
+            .collect();
+        println!("skip_keys:{:?}", skip_keys);
         JsonSolver {
             expression,
             pretty,
             recursive,
             json_line,
             skip_empty,
+            skip_keys,
         }
     }
 }
@@ -35,36 +43,40 @@ impl JsonSolver {
     where
         R: std::io::BufRead,
     {
-        Ok(value
+        value
             .lines()
             .map(|value| value.map(|v| self.resolve_value_impl(&v)))
             .flatten()
             .flatten()
             .flatten()
-            .map(|s| format!("{}", JsonSolver::value_to_string(self.pretty, &s)))
-            .for_each(|s| println!("{}", s)))
+            .map(|s| JsonSolver::value_to_string(self.pretty, &s))
+            .for_each(|s| println!("{}", s));
+        Ok(())
     }
 
-    pub fn resolve_value<'a>(&self, value: &'a str) -> Result<String, TError> {
-        let lines = if self.json_line {
-            value.split('\n').filter(|v| !v.is_empty()).collect()
-        } else {
-            vec![value]
-        };
-        Ok(lines
-            .into_iter()
+    pub fn resolve_value(&self, value: &str) -> Result<Vec<String>, TError> {
+        Ok(value
+            .split('\n')
+            .filter(|v| !v.is_empty())
             .map(|value| self.resolve_value_impl(value))
             .collect::<Result<Vec<_>, _>>()?
             .into_iter()
             .flatten()
-            .map(|s| format!("{}", JsonSolver::value_to_string(self.pretty, &s)))
-            .collect::<Vec<String>>()
-            .join("\n"))
+            .map(|s| JsonSolver::value_to_string(self.pretty, &s))
+            .collect::<Vec<String>>())
     }
 
-    fn resolve_value_impl<'a>(&self, value: &'a str) -> Result<Vec<serde_json::Value>, TError> {
-        let root_proto = serde_json::from_str::<serde_json::Value>(value)?;
-        let root = self.recursively_parse(root_proto)?;
+    pub fn resolve_line(&self, value: &str) -> Result<Vec<String>, TError> {
+        let result = self.resolve_value_impl(value)?;
+        Ok(result
+            .iter()
+            .map(|v| JsonSolver::value_to_string(self.pretty, v))
+            .collect())
+    }
+
+    fn resolve_value_impl(&self, value: &str) -> Result<Vec<serde_json::Value>, TError> {
+        let root = serde_json::from_str::<serde_json::Value>(value)
+            .map(|s| self.recursively_parse(s).map(|s| self.remove_keys(s)))??;
         let resolved_value = {
             let mut result = vec![root];
             for expr in &self.expression {
@@ -81,12 +93,10 @@ impl JsonSolver {
                                             let result = values.get(expr.as_str()).cloned();
                                             if let Some(v) = result {
                                                 Ok(Some(v))
+                                            } else if self.skip_empty {
+                                                Ok(None)
                                             } else {
-                                                if self.skip_empty {
-                                                    Ok(None)
-                                                } else {
-                                                    Err(TError::KeyNotExist(expr.clone()))
-                                                }
+                                                Err(TError::KeyNotExist(expr.clone()))
                                             }
                                         });
                                         Box::new(next)
@@ -96,12 +106,10 @@ impl JsonSolver {
                                             let result = o.get(expr.as_str()).cloned();
                                             if let Some(v) = result {
                                                 Ok(Some(v))
+                                            } else if self.skip_empty {
+                                                Ok(None)
                                             } else {
-                                                if self.skip_empty {
-                                                    Ok(None)
-                                                } else {
-                                                    Err(TError::KeyNotExist(expr.clone()))
-                                                }
+                                                Err(TError::KeyNotExist(expr.clone()))
                                             }
                                         });
                                         Box::new(next)
@@ -112,7 +120,7 @@ impl JsonSolver {
                         .flatten()
                         .collect::<Result<Vec<_>, _>>()?
                         .into_iter()
-                        .filter_map(|e| e)
+                        .flatten()
                         .collect::<Vec<_>>();
             }
             result
@@ -120,7 +128,7 @@ impl JsonSolver {
         Ok(resolved_value)
     }
 
-    fn recursively_parse<'a>(&self, value: serde_json::Value) -> Result<serde_json::Value, TError> {
+    fn recursively_parse(&self, value: serde_json::Value) -> Result<serde_json::Value, TError> {
         if self.recursive {
             match value {
                 serde_json::Value::Array(v) => {
@@ -131,7 +139,7 @@ impl JsonSolver {
                     Ok(serde_json::Value::Array(result))
                 }
                 serde_json::Value::String(s) => {
-                    Ok(serde_json::from_str(&s).unwrap_or_else(|_| serde_json::Value::String(s)))
+                    Ok(serde_json::from_str(&s).unwrap_or(serde_json::Value::String(s)))
                 }
                 serde_json::Value::Object(map) => {
                     let result = map
@@ -150,6 +158,19 @@ impl JsonSolver {
         }
     }
 
+    fn remove_keys(&self, value: serde_json::Value) -> serde_json::Value {
+        match value {
+            serde_json::Value::Object(map) => {
+                let result = map
+                    .into_iter()
+                    .filter(|s| !self.skip_keys.contains(&s.0))
+                    .collect::<serde_json::Map<_, _>>();
+                serde_json::Value::Object(result)
+            }
+            v => v,
+        }
+    }
+
     fn value_to_string(pretty: bool, value: &serde_json::Value) -> String {
         if pretty {
             serde_json::to_string_pretty(value).unwrap()
@@ -195,5 +216,15 @@ pub fn clap_app() -> clap::App<'static, 'static> {
                 .help("If expression fails to resolve, skip the value")
                 .takes_value(false),
         )
+        .arg(
+            clap::Arg::with_name("skip-key")
+                .long("skip-key")
+                .help(
+                    "Keys to be removed from the final output. \
+                    Currently only supports removing root keys.",
+                )
+                .takes_value(true)
+                .multiple(true),
+        )
         .author(clap::crate_authors!())
 }
